package com.qr.flutter_toolkit.video_thumbnail;

import android.annotation.SuppressLint;
import android.graphics.Bitmap;
import android.util.Log;

import androidx.annotation.NonNull;

import com.bumptech.glide.Glide;
import com.bumptech.glide.request.FutureTarget;

import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.util.Map;
import java.util.concurrent.ExecutionException;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;
import io.reactivex.Observable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.schedulers.Schedulers;

public class VideoThumbnailHandler implements MethodChannel.MethodCallHandler {
    private static final String TAG = VideoThumbnailHandler.class.getSimpleName();

    private final PluginRegistry.Registrar registrar;

    public VideoThumbnailHandler(PluginRegistry.Registrar registrar) {
        this.registrar = registrar;
    }

    @SuppressLint("CheckResult")
    @Override
    public void onMethodCall(@NonNull MethodCall methodCall, @NonNull MethodChannel.Result result) {
        final Map<String, Object> args = methodCall.arguments();
        final String video = (String) args.get("video");
        final int format = (int) args.get("format");
        final int quality = (int) args.get("quality");
        if (methodCall.method.equals("file")) {
            Observable.fromCallable(() -> {
                return buildThumbnailFile(video, format, quality);
            }).subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(s -> {
                        result.success(s);
                    }, throwable -> {
                        result.error("exception", throwable.getMessage(), null);
                    });
        } else if (methodCall.method.equals("data")) {
            Observable.fromCallable(() -> {
                return buildThumbnailData(video, format, quality);
            }).subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(bytes -> {
                        result.success(bytes);
                    }, throwable -> {
                        result.error("exception", throwable.getMessage(), null);
                    });
        } else {
            result.notImplemented();
        }
    }

    private static Bitmap.CompressFormat intToFormat(int format) {
        switch (format) {
            default:
            case 0:
                return Bitmap.CompressFormat.JPEG;
            case 1:
                return Bitmap.CompressFormat.PNG;
            case 2:
                return Bitmap.CompressFormat.WEBP;
        }
    }

    private static String formatExt(int format) {
        switch (format) {
            default:
            case 0:
                return "jpg";
            case 1:
                return "png";
            case 2:
                return "webp";
        }
    }

    private byte[] buildThumbnailData(String vidPath, int format, int quality) throws ExecutionException, InterruptedException {
        FutureTarget<Bitmap> submit = Glide.with(registrar.activeContext()).asBitmap().load(vidPath).submit();

        Bitmap bitmap = submit.get();
        if (bitmap == null)
            throw new NullPointerException();

        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        bitmap.compress(intToFormat(format), quality, stream);
        bitmap.recycle();

        Glide.with(registrar.activeContext()).clear(submit);
        return stream.toByteArray();
    }

    private String buildThumbnailFile(String vidPath, int format, int quality) throws ExecutionException, InterruptedException {
        Log.d(TAG, String.format("buildThumbnailFile( format:%d, quality:%d )", format, quality));
        final byte[] bytes = buildThumbnailData(vidPath, format, quality);
        final String ext = formatExt(format);
        final int i = vidPath.lastIndexOf(".");
        String fullpath = vidPath.substring(0, i + 1) + ext;

        try {
            FileOutputStream f = new FileOutputStream(fullpath);
            f.write(bytes);
            f.close();
            Log.d(TAG, String.format("buildThumbnailFile( written:%d )", bytes.length));
        } catch (java.io.IOException e) {
            e.getStackTrace();
            throw new RuntimeException(e);
        }
        return fullpath;
    }
}
